"""This file contains global constants."""

NAME = "gcipher"
VERSION = "1.0"
COPYRIGHT = "Copyright (C) 2003 Shannon -jj Behrens.  All rights reserved."
INFO = """\
a simple encryption tool

Read Help/Contents to learn how to use gcipher
as a command line filter or as a network proxy."""
AUTHORS = ["Shannon -jj Behrens <jjinux@yahoo.com>"]
GLADEDIR = "/usr/share/gcipher/lib"
